#!/usr/bin/env python

#
# Draw the Batman curve from the Wolfram website. The curves are f(x)
# and g(x). Save curves in files f.out and g.out. Compute area in
# different ways: with simple Monte Carlo, with trapezoida rule, in a
# naive way.
#

from math import pi
from cmath import sqrt
from cmath import acos
from numpy import arange
from numpy import zeros
from sympy.functions.special.delta_functions import Heaviside
from random import uniform


def w(x):
    return 3.0 * sqrt(1.0 - (x / 7.0)**2)


def l(x):
    return 0.5 * (x + 3.0) - 3.0 / 7.0 * sqrt(10.0) * sqrt(4.0 - (
        x + 1)**2) + 6.0 / 7.0 * sqrt(10.0)


def h(x):
    return 0.5 * (3.0 * (abs(x + 0.5) + abs(x - 0.5) + 6.0) - 11.0 *
                  (abs(x + 3.0 / 4.0) + abs(x - 3.0 / 4.0)))


def r(x):
    return 0.5 * (3.0 - x) - 3.0 / 7.0 * sqrt(10.0) * sqrt(4.0 - (
        x - 1.0)**2) + 6.0 / 7.0 * sqrt(10.0)


def H(x):
    return Heaviside(x)


def f(x):
    return (h(x) - l(x)) * H(x + 1.0) + (r(x) - h(x)) * H(x - 1.0) + (
        l(x) - w(x)) * H(x + 3.0) + (w(x) - r(x)) * H(x - 3.0) + w(x)


def g(x):
    return 0.5 * (
        abs(0.5 * x) + sqrt(1.0 - (abs(abs(x) - 2.0) - 1)**2) - 1.0 / 112.0 *
        (3.0 * sqrt(33.0) -
         7.0) * x**2 + 3.0 * sqrt(1.0 - (1.0 / 7.0 * x)**2) - 3.0) * (
             (x + 4.0) / abs(x + 4.0) -
             (x - 4.0) / abs(x - 4.0)) - 3.0 * sqrt(1.0 - (1.0 / 7.0 * x)**2)


d = 0.001
ax = arange(-7.0, 7.0, d)
af = zeros(len(ax))
ag = zeros(len(ax))
for i, x in enumerate(ax):
    af[i] = complex(f(x)).real
    ag[i] = complex(g(x)).real

a = open("f.out", "w")
b = open("g.out", "w")
for i, x in enumerate(ax):
    a.write("%s  %s\n" % (x, af[i]))
    b.write("%s  %s\n" % (x, ag[i].real))
a.close()
b.close()

q = 0.0
p = 0.0
for i, x in enumerate(ax):
    if (i == 0):
        continue
    j = i - 1
    q = q + (af[j] + af[i]) * 0.5 + abs(ag[j] + ag[i]) * 0.5
    p = p + (af[i] - ag[i])
q = q * d
p = p * d
print "Approximated area (naive) : ", q
print "Approximated area (trapez): ", p

A = 955.0 / 48.0 - 2.0 / 7.0 * (2 * sqrt(33.0) + 7.0 * pi + 3.0 * sqrt(10.0) *
                                (pi - 1.0)) + 21.0 * (acos(3.0 / 7.0) + acos(
                                    4.0 / 7.0))

n = 10000
m = 0

a = open("hits.out", "w")
b = open("miss.out", "w")
for i in xrange(n):
    if (i % 1000 == 0):
        print "%s / %s" % (i, n)
    x = uniform(-7.0, 7.0)
    y = uniform(-4.0, 4.0)
    fx = complex(f(x)).real
    gx = complex(g(x)).real
    if (fx >= y and gx <= y):
        a.write("%s  %s\n" % (x, y))
        m = m + 1
    else:
        b.write("%s  %s\n" % (x, y))
a.close()
b.close()

print "Exact area:        ", A.real
print "Approximated area: ", m / float(n) * 14 * 8
